package ch.zhaw.pm2.racetrack.strategy;

import ch.zhaw.pm2.racetrack.PositionVector.Direction;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * This Class represent the MoveListStrategy. The Directions returned by the
 * nextMove() are written down in a List.
 *
 */
public class MoveListStrategy implements MoveStrategy {

    private final List<Direction> moveList;
    private int pointer;

    public MoveListStrategy(File moveListFile) throws FileNotFoundException{
        moveList = new ArrayList<>();
        pointer = -1;
        readFile(moveListFile);
    }

    /**
     * This Method will read in a File and checks line by line if the file contains a valid direction.
     * If so the direction will be added to the moveList.
     * @param trackFile the file to read in the directions
     * @throws FileNotFoundException if the file does not exist.
     */
    private void readFile(File trackFile) throws FileNotFoundException {
        Scanner scanner = new Scanner(new FileInputStream(trackFile), StandardCharsets.UTF_8);
        Direction[] directions = Direction.values();
        while (scanner.hasNextLine()) {
            String line = scanner.nextLine();
            for (Direction direction : directions) {
                if (direction.toString().equals(line)) {
                    moveList.add(direction);
                    break;
                }
            }
        }
    }

    /**
     * This method will be used to return the next Direction for the car.
     * @return the next direction from the list. Returns null if the list is empty.
     */
    @Override
    public Direction nextMove() {
        pointer += 1;
        if (pointer < moveList.size()) {
            return moveList.get(pointer);
        }
        return null;
    }
}
